MODULE limrst
   !!======================================================================
   !!                     ***  MODULE  limrst  ***
   !! Ice restart :  write the ice restart file
   !!======================================================================
   !! History:   -   ! 2005-04 (M. Vancoppenolle) Original code
   !!           3.0  ! 2008-03 (C. Ethe) restart files in using IOM interface
   !!           4.0  ! 2011-02 (G. Madec) dynamical allocation
   !!----------------------------------------------------------------------
#if defined key_lim3
   !!----------------------------------------------------------------------
   !!   'key_lim3' :                                   LIM sea-ice model
   !!----------------------------------------------------------------------
   !!   lim_rst_opn   : open ice restart file
   !!   lim_rst_write : write of the restart file 
   !!   lim_rst_read  : read  the restart file 
   !!----------------------------------------------------------------------
   USE ice            ! sea-ice variables
   USE oce     , ONLY :  snwice_mass, snwice_mass_b
   USE dom_oce        ! ocean domain
   USE sbc_oce        ! Surface boundary condition: ocean fields
   USE sbc_ice        ! Surface boundary condition: ice fields
   USE in_out_manager ! I/O manager
   USE iom            ! I/O library
   USE lib_mpp        ! MPP library
   USE wrk_nemo       ! work arrays
   USE lib_fortran    ! Fortran utilities (allows no signed zero when 'key_nosignedzero' defined)  
   USE limctl

   IMPLICIT NONE
   PRIVATE

   PUBLIC   lim_rst_opn    ! routine called by icestep.F90
   PUBLIC   lim_rst_write  ! routine called by icestep.F90
   PUBLIC   lim_rst_read   ! routine called by sbc_lim_init

   LOGICAL, PUBLIC ::   lrst_ice         !: logical to control the ice restart write 
   INTEGER, PUBLIC ::   numrir, numriw   !: logical unit for ice restart (read and write)

   !!----------------------------------------------------------------------
   !! NEMO/LIM3 4.0 , UCL - NEMO Consortium (2011)
   !! $Id$
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE lim_rst_opn( kt )
      !!----------------------------------------------------------------------
      !!                    ***  lim_rst_opn  ***
      !!
      !! ** purpose  :   output of sea-ice variable in a netcdf file
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt       ! number of iteration
      !
      CHARACTER(len=20)   ::   clkt     ! ocean time-step define as a character
      CHARACTER(len=50)   ::   clname   ! ice output restart file name
      CHARACTER(len=256)  ::   clpath   ! full path to ice output restart file 
      !!----------------------------------------------------------------------
      !
      IF( kt == nit000 )   lrst_ice = .FALSE.   ! default definition

      ! in order to get better performances with NetCDF format, we open and define the ice restart file 
      ! one ice time step before writing the data (-> at nitrst - 2*nn_fsbc + 1), except if we write ice 
      ! restart files every ice time step or if an ice restart file was writen at nitend - 2*nn_fsbc + 1
      IF( kt == nitrst - 2*nn_fsbc + 1 .OR. nstock == nn_fsbc    &
         &                             .OR. ( kt == nitend - nn_fsbc + 1 .AND. .NOT. lrst_ice ) ) THEN
         IF( nitrst <= nitend .AND. nitrst > 0 ) THEN
            ! beware of the format used to write kt (default is i8.8, that should be large enough...)
            IF( nitrst > 99999999 ) THEN   ;   WRITE(clkt, *       ) nitrst
            ELSE                           ;   WRITE(clkt, '(i8.8)') nitrst
            ENDIF
            ! create the file
            clname = TRIM(cexper)//"_"//TRIM(ADJUSTL(clkt))//"_"//TRIM(cn_icerst_out)
            clpath = TRIM(cn_icerst_outdir) 
            IF( clpath(LEN_TRIM(clpath):) /= '/' ) clpath = TRIM(clpath)//'/'
            IF(lwp) THEN
               WRITE(numout,*)
               SELECT CASE ( jprstlib )
               CASE DEFAULT
                  WRITE(numout,*) '             open ice restart NetCDF file: ',TRIM(clpath)//clname
               END SELECT
               IF( kt == nitrst - 2*nn_fsbc + 1 ) THEN   
                  WRITE(numout,*)         '             kt = nitrst - 2*nn_fsbc + 1 = ', kt,' date= ', ndastp
               ELSE   ;   WRITE(numout,*) '             kt = '                         , kt,' date= ', ndastp
               ENDIF
            ENDIF
            !
            CALL iom_open( TRIM(clpath)//TRIM(clname), numriw, ldwrt = .TRUE., kiolib = jprstlib, ndlev = jpl )
            lrst_ice = .TRUE.
         ENDIF
      ENDIF
      !
      IF( ln_limctl )   CALL lim_prt( kt, iiceprt, jiceprt, 1, ' - Beginning the time step - ' )   ! control print
   END SUBROUTINE lim_rst_opn


   SUBROUTINE lim_rst_write( kt )
      !!----------------------------------------------------------------------
      !!                    ***  lim_rst_write  ***
      !!
      !! ** purpose  :   output of sea-ice variable in a netcdf file
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt     ! number of iteration
      !!
      INTEGER ::   ji, jj, jk ,jl   ! dummy loop indices
      INTEGER ::   iter
      CHARACTER(len=25) ::   znam
      CHARACTER(len=2)  ::   zchar, zchar1
      REAL(wp), POINTER, DIMENSION(:,:, :) :: z3d
      !!----------------------------------------------------------------------

      CALL wrk_alloc( jpi, jpj, jpl,  z3d )

      iter = kt + nn_fsbc - 1   ! ice restarts are written at kt == nitrst - nn_fsbc + 1

      IF( iter == nitrst ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'lim_rst_write : write ice restart file  kt =', kt
         IF(lwp) WRITE(numout,*) '~~~~~~~~~~~~~~'         
      ENDIF

      ! Write in numriw (if iter == nitrst)
      ! ------------------ 
      !                                                                        ! calendar control
      CALL iom_rstput( iter, nitrst, numriw, 'nn_fsbc', REAL( nn_fsbc, wp ) )      ! time-step 
      CALL iom_rstput( iter, nitrst, numriw, 'kt_ice' , REAL( iter   , wp ) )      ! date

      ! Prognostic variables 
      znam = 'v_i'
      CALL iom_rstput( iter, nitrst, numriw, znam , v_i )
      znam = 'v_s'
      CALL iom_rstput( iter, nitrst, numriw, znam , v_s )
      znam = 'smv_i'
      CALL iom_rstput( iter, nitrst, numriw, znam , smv_i )
      znam = 'oa_i'
      CALL iom_rstput( iter, nitrst, numriw, znam , oa_i )
      znam = 'a_i'
      CALL iom_rstput( iter, nitrst, numriw, znam , a_i )
      znam = 't_su'
      CALL iom_rstput( iter, nitrst, numriw, znam , t_su )
      znam = 'tempt_sl1'
      z3d = e_s(:,:,1,:)
      CALL iom_rstput( iter, nitrst, numriw, znam , z3d )
      DO jk = 1, nlay_i 
         WRITE(zchar1,'(I2.2)') jk
         znam = 'tempt'//'_il'//zchar1
         z3d(:,:,:) = e_i(:,:,jk,:)
         CALL iom_rstput( iter, nitrst, numriw, znam , z3d )
      END DO

      CALL iom_rstput( iter, nitrst, numriw, 'u_ice'        , u_ice      )
      CALL iom_rstput( iter, nitrst, numriw, 'v_ice'        , v_ice      )
      CALL iom_rstput( iter, nitrst, numriw, 'stress1_i'    , stress1_i  )
      CALL iom_rstput( iter, nitrst, numriw, 'stress2_i'    , stress2_i  )
      CALL iom_rstput( iter, nitrst, numriw, 'stress12_i'   , stress12_i )
      CALL iom_rstput( iter, nitrst, numriw, 'snwice_mass'  , snwice_mass )
      CALL iom_rstput( iter, nitrst, numriw, 'snwice_mass_b', snwice_mass_b )

      ! In case Prather scheme is used for advection, write second order moments
      ! ------------------------------------------------------------------------
      IF( nn_limadv == -1 ) THEN
         write(*,*) 'BACK in LIM3+1',nn_limadv 
         znam = 'sxice'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxice )
         znam = 'syice'
         CALL iom_rstput( iter, nitrst, numriw, znam , syice )
         znam = 'sxxice'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxxice )
         znam = 'syyice'
         CALL iom_rstput( iter, nitrst, numriw, znam , syyice )
         znam = 'sxyice'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxyice )
         znam = 'sxsn'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxsn )
         znam = 'sysn'
         CALL iom_rstput( iter, nitrst, numriw, znam , sysn )
         znam = 'sxxsn'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxxsn )
         znam = 'syysn'
         CALL iom_rstput( iter, nitrst, numriw, znam , syysn )
         znam = 'sxysn'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxysn )
         znam = 'sxa'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxa )
         znam = 'sya'
         CALL iom_rstput( iter, nitrst, numriw, znam , sya )
         znam = 'sxxa'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxxa )
         znam = 'syya'
         CALL iom_rstput( iter, nitrst, numriw, znam , syya )
         znam = 'sxya'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxya )
         znam = 'sxc0'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxc0 )
         znam = 'syc0'
         CALL iom_rstput( iter, nitrst, numriw, znam , syc0 )
         znam = 'sxxc0'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxxc0 )
         znam = 'syyc0'
         CALL iom_rstput( iter, nitrst, numriw, znam , syyc0 )
         znam = 'sxyc0'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxyc0 )
         znam = 'sxsal'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxsal )
         znam = 'sysal'
         CALL iom_rstput( iter, nitrst, numriw, znam , sysal )
         znam = 'sxxsal'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxxsal )
         znam = 'syysal'
         CALL iom_rstput( iter, nitrst, numriw, znam , syysal )
         znam = 'sxysal'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxysal )
         znam = 'sxage'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxage )
         znam = 'syage'
         CALL iom_rstput( iter, nitrst, numriw, znam , syage )
         znam = 'sxxage'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxxage )
         znam = 'syyage'
         CALL iom_rstput( iter, nitrst, numriw, znam , syyage )
         znam = 'sxyage'
         CALL iom_rstput( iter, nitrst, numriw, znam , sxyage )

         CALL iom_rstput( iter, nitrst, numriw, 'sxopw ' ,  sxopw  )
         CALL iom_rstput( iter, nitrst, numriw, 'syopw ' ,  syopw  )
         CALL iom_rstput( iter, nitrst, numriw, 'sxxopw' ,  sxxopw )
         CALL iom_rstput( iter, nitrst, numriw, 'syyopw' ,  syyopw )
         CALL iom_rstput( iter, nitrst, numriw, 'sxyopw' ,  sxyopw )
         
         DO jk = 1, nlay_i 
               WRITE(zchar1,'(I2.2)') jk
               znam = 'sxe'//'_il'//zchar1
               z3d(:,:,:) = sxe(:,:,jk,:)
               CALL iom_rstput( iter, nitrst, numriw, znam , z3d )
               znam = 'sye'//'_il'//zchar1
               z3d(:,:,:) = sye(:,:,jk,:)
               CALL iom_rstput( iter, nitrst, numriw, znam , z3d )
               znam = 'sxxe'//'_il'//zchar1
               z3d(:,:,:) = sxxe(:,:,jk,:)
               CALL iom_rstput( iter, nitrst, numriw, znam , z3d )
               znam = 'syye'//'_il'//zchar1
               z3d(:,:,:) = syye(:,:,jk,:)
               CALL iom_rstput( iter, nitrst, numriw, znam , z3d )
               znam = 'sxye'//'_il'//zchar1
               z3d(:,:,:) = sxye(:,:,jk,:)
               CALL iom_rstput( iter, nitrst, numriw, znam , z3d )
         END DO

      ENDIF
      
      ! close restart file
      ! ------------------
      IF( iter == nitrst ) THEN
         CALL iom_close( numriw )
         lrst_ice = .FALSE.
      ENDIF
      !
      CALL wrk_dealloc( jpi, jpj, jpl, z3d )
      !
   END SUBROUTINE lim_rst_write


   SUBROUTINE lim_rst_read
      !!----------------------------------------------------------------------
      !!                    ***  lim_rst_read  ***
      !!
      !! ** purpose  :   read of sea-ice variable restart in a netcdf file
      !!----------------------------------------------------------------------
      INTEGER :: ji, jj, jk, jl
      REAL(wp) ::   zfice, ziter
      REAL(wp), POINTER, DIMENSION(:,:, :) ::   z3d
      CHARACTER(len=25) ::   znam
      CHARACTER(len=2)  ::   zchar, zchar1
      INTEGER           ::   jlibalt = jprstlib
      LOGICAL           ::   llok
      !!----------------------------------------------------------------------

      CALL wrk_alloc( jpi, jpj, jpl, z3d )

      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'lim_rst_read : read ice NetCDF restart file'
         WRITE(numout,*) '~~~~~~~~~~~~~'
      ENDIF

      CALL iom_open ( TRIM(cn_icerst_indir)//'/'//cn_icerst_in, numrir, kiolib = jprstlib, ndlev = jpl )

      CALL iom_get( numrir, 'nn_fsbc', zfice )
      CALL iom_get( numrir, 'kt_ice' , ziter )    
      IF(lwp) WRITE(numout,*) '   read ice restart file at time step    : ', ziter
      IF(lwp) WRITE(numout,*) '   in any case we force it to nit000 - 1 : ', nit000 - 1

      !Control of date

      IF( ( nit000 - NINT(ziter) ) /= 1 .AND. ABS( nrstdt ) == 1 )   &
         &     CALL ctl_stop( 'lim_rst_read ===>>>> : problem with nit000 in ice restart',  &
         &                   '   verify the file or rerun with the value 0 for the',        &
         &                   '   control of time parameter  nrstdt' )
      IF( NINT(zfice) /= nn_fsbc          .AND. ABS( nrstdt ) == 1 )   &
         &     CALL ctl_stop( 'lim_rst_read ===>>>> : problem with nn_fsbc in ice restart',  &
         &                   '   verify the file or rerun with the value 0 for the',         &
         &                   '   control of time parameter  nrstdt' )

      ! Prognostic variables 
      znam = 'v_i'
      CALL iom_get( numrir, jpdom_autoglo, znam , v_i )
      znam = 'v_s'
      CALL iom_get( numrir, jpdom_autoglo, znam , v_s )
      znam = 'smv_i'
      CALL iom_get( numrir, jpdom_autoglo, znam , smv_i )
      znam = 'oa_i'
      CALL iom_get( numrir, jpdom_autoglo, znam , oa_i )
      znam = 'a_i'
      CALL iom_get( numrir, jpdom_autoglo, znam , a_i )
      znam = 't_su'
      CALL iom_get( numrir, jpdom_autoglo, znam , t_su )
      znam = 'tempt_sl1'
      CALL iom_get( numrir, jpdom_autoglo, znam , z3d )
      e_s(:,:,1,:) = z3d
      DO jk = 1, nlay_i 
         WRITE(zchar1,'(I2.2)') jk
         znam = 'tempt'//'_il'//zchar1
         CALL iom_get( numrir, jpdom_autoglo, znam , z3d )
         e_i(:,:,jk,:) = z3d(:,:,:)
      END DO

      CALL iom_get( numrir, jpdom_autoglo, 'u_ice'     , u_ice      )
      CALL iom_get( numrir, jpdom_autoglo, 'v_ice'     , v_ice      )
      CALL iom_get( numrir, jpdom_autoglo, 'stress1_i' , stress1_i  )
      CALL iom_get( numrir, jpdom_autoglo, 'stress2_i' , stress2_i  )
      CALL iom_get( numrir, jpdom_autoglo, 'stress12_i', stress12_i )
      CALL iom_get( numrir, jpdom_autoglo, 'snwice_mass'  , snwice_mass )
      CALL iom_get( numrir, jpdom_autoglo, 'snwice_mass_b', snwice_mass_b )

      ! In case Prather scheme is used for advection, read second order moments
      ! ------------------------------------------------------------------------
      IF( nn_limadv == -1 ) THEN

         znam = 'sxice'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxice )
         znam = 'syice'
         CALL iom_get( numrir, jpdom_autoglo, znam , syice )
         znam = 'sxxice'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxxice )
         znam = 'syyice'
         CALL iom_get( numrir, jpdom_autoglo, znam , syyice )
         znam = 'sxyice'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxyice )
         znam = 'sxsn'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxsn )
         znam = 'sysn'
         CALL iom_get( numrir, jpdom_autoglo, znam , sysn )
         znam = 'sxxsn'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxxsn )
         znam = 'syysn'
         CALL iom_get( numrir, jpdom_autoglo, znam , syysn )
         znam = 'sxysn'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxysn )
         znam = 'sxa'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxa )
         znam = 'sya'
         CALL iom_get( numrir, jpdom_autoglo, znam , sya )
         znam = 'sxxa'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxxa )
         znam = 'syya'
         CALL iom_get( numrir, jpdom_autoglo, znam , syya )
         znam = 'sxya'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxya )
         znam = 'sxc0'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxc0 )
         znam = 'syc0'
         CALL iom_get( numrir, jpdom_autoglo, znam , syc0 )
         znam = 'sxxc0'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxxc0 )
         znam = 'syyc0'
         CALL iom_get( numrir, jpdom_autoglo, znam , syyc0 )
         znam = 'sxyc0'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxyc0 )
         znam = 'sxsal'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxsal )
         znam = 'sysal'
         CALL iom_get( numrir, jpdom_autoglo, znam , sysal )
         znam = 'sxxsal'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxxsal )
         znam = 'syysal'
         CALL iom_get( numrir, jpdom_autoglo, znam , syysal )
         znam = 'sxysal'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxysal )
         znam = 'sxage'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxage )
         znam = 'syage'
         CALL iom_get( numrir, jpdom_autoglo, znam , syage )
         znam = 'sxxage'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxxage )
         znam = 'syyage'
         CALL iom_get( numrir, jpdom_autoglo, znam , syyage )
         znam = 'sxyage'
         CALL iom_get( numrir, jpdom_autoglo, znam , sxyage )

         CALL iom_get( numrir, jpdom_autoglo, 'sxopw ' ,  sxopw  )
         CALL iom_get( numrir, jpdom_autoglo, 'syopw ' ,  syopw  )
         CALL iom_get( numrir, jpdom_autoglo, 'sxxopw' ,  sxxopw )
         CALL iom_get( numrir, jpdom_autoglo, 'syyopw' ,  syyopw )
         CALL iom_get( numrir, jpdom_autoglo, 'sxyopw' ,  sxyopw )

         DO jl = 1, jpl 
            WRITE(zchar,'(I2.2)') jl
            DO jk = 1, nlay_i 
               WRITE(zchar1,'(I2.2)') jk
               znam = 'sxe'//'_il'//zchar1
               CALL iom_get( numrir, jpdom_autoglo, znam , z3d )
               sxe(:,:,jk,:) = z3d(:,:,:)
               znam = 'sye'//'_il'//zchar1
               CALL iom_get( numrir, jpdom_autoglo, znam , z3d )
               sye(:,:,jk,:) = z3d(:,:,:)
               znam = 'sxxe'//'_il'//zchar1
               CALL iom_get( numrir, jpdom_autoglo, znam , z3d )
               sxxe(:,:,jk,:) = z3d(:,:,:)
               znam = 'syye'//'_il'//zchar1
               CALL iom_get( numrir, jpdom_autoglo, znam , z3d )
               syye(:,:,jk,:) = z3d(:,:,:)
               znam = 'sxye'//'_il'//zchar1
               CALL iom_get( numrir, jpdom_autoglo, znam , z3d )
               sxye(:,:,jk,:) = z3d(:,:,:)
            END DO
         END DO
         !
      END IF
      
      ! clem: I do not understand why the following IF is needed
      !       I suspect something inconsistent in the main code with option nn_icesal=1
      IF( nn_icesal == 1 ) THEN
         DO jl = 1, jpl 
            sm_i(:,:,jl) = rn_icesal
            DO jk = 1, nlay_i 
               s_i(:,:,jk,jl) = rn_icesal
            END DO
         END DO
      ENDIF
      !
      !CALL iom_close( numrir ) !clem: closed in sbcice_lim.F90
      !
      CALL wrk_dealloc( jpi, jpj, jpl, z3d )
      !
   END SUBROUTINE lim_rst_read

#else
   !!----------------------------------------------------------------------
   !!   Default option :       Empty module            NO LIM sea-ice model
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE lim_rst_read             ! Empty routine
   END SUBROUTINE lim_rst_read
   SUBROUTINE lim_rst_write            ! Empty routine
   END SUBROUTINE lim_rst_write
#endif

   !!======================================================================
END MODULE limrst
